import torchvision.transforms as T

def image_preprocess(image, resize_dim=256):
    preprocess = T.Compose([
            T.Resize(resize_dim),
            T.CenterCrop(224),
            T.ToTensor(),
            T.Normalize(mean=[0.485, 0.456, 0.406],
                        std=[0.229, 0.224, 0.225]),
    ])
    
    return preprocess(image)


import os
from PIL import Image

def DataLoader(path):
    images = []
    for file in os.listdir(path):
        f_img = path+"/"+file
        img = Image.open(f_img)
        images.append(img)
    
    return images


import torchvision.models as models

def Model(pretrained_model):
    model = getattr(models, '%s' %pretrained_model)(pretrained=True)    
    # Set the model to Evaluation mode
    model.eval()
    
    return model


import torch
from feacher import preprocess as ps
from feacher import model as ml

def feature_vector(pretrained_model, image, resize_dim, layer):
    model = ml.Model(pretrained_model=pretrained_model)
    input_image = ps.image_preprocess(image, resize_dim)
    input_batch = input_image.unsqueeze(0)

    with torch.no_grad():
        feature_vector = None
        
        def hook(module_, input_, feature_vector_):
            nonlocal feature_vector
            feature_vector = feature_vector_

        a_hook = getattr(model, '%s' %layer).register_forward_hook(hook)        
        model(input_batch)
        a_hook.remove()
        
        return feature_vector



from feacher import feature_extractor as fe
from feacher import dataloader as dl

def extract(path, pretrained_model='resnet18', resize_dim=256, layer=None):

    images = dl.DataLoader(path)
    image_features = []

    if(layer == None):
        layer = 'avgpool'

    for image in images:
        extracted_features = fe.feature_vector(pretrained_model,
                                               image,
                                               resize_dim,
                                               layer)
        image_features.append(extracted_features)

    print(f"\n\n=============================================")
    print(f"Feacher (@qpochlabs) - Parameters and Results")
    print(f"=============================================")
    print(f"Model         : {pretrained_model}")
    print(f"Layer         : {layer}")
    print(f"feature size  : {image_features[0].shape}")
    print(f"=============================================\n\n")

    return image_features